@@ -88,8 +88,9 @@ def __get_connection_SNS():
             logger.debug(
                 'Authenticating to SNS using '
                 'credentials in configuration file')
+            region = get_global_option('region')
             connection = sns.connect_to_region(
-                get_global_option('region'),
+                region,
                 aws_access_key_id=get_global_option(
                     'aws_access_key_id'),
                 aws_secret_access_key=get_global_option(
@@ -99,14 +100,16 @@ def __get_connection_SNS():
                 logger.debug(
                     'Authenticating to SNS using EC2 instance profile')
                 metadata = get_instance_metadata(timeout=1, num_retries=1)
+                region = metadata['placement']['availability-zone'][:-1]
                 connection = sns.connect_to_region(
-                    metadata['placement']['availability-zone'][:-1],
+                    region,
                     profile_name=metadata['iam']['info'][u'InstanceProfileArn'])
             except KeyError:
                 logger.debug(
                     'Authenticating to SNS using '
                     'env vars / boto configuration')
-                connection = sns.connect_to_region(get_global_option('region'))
+                region = get_global_option('region')
+                connection = sns.connect_to_region(region)
 
     except Exception as err:
         logger.error('Failed connecting to SNS: {0}'.format(err))
@@ -115,7 +118,7 @@ def __get_connection_SNS():
             'https://github.com/sebdah/dynamic-dynamodb/issues')
         raise
 
-    logger.debug('Connected to SNS in {0}'.format(get_global_option('region')))
+    logger.debug('Connected to SNS in {0}'.format(region))
     return connection
 
 SNS_CONNECTION = __get_connection_SNS()
