@@ -10,24 +10,44 @@ from dynamic_dynamodb.log_handler import LOGGER as logger
 from dynamic_dynamodb.config_handler import get_table_option, get_global_option
 
 
-def ensure_provisioning(table_name, key_name):
+def ensure_provisioning(
+        table_name, key_name,
+        num_consec_read_checks,
+        num_consec_write_checks):
     """ Ensure that provisioning is correct
 
     :type table_name: str
     :param table_name: Name of the DynamoDB table
     :type key_name: str
     :param key_name: Configuration option key name
+    :type num_consec_read_checks: int
+    :param num_consec_read_checks: How many consecutive checks have we had
+    :type num_consec_write_checks: int
+    :param num_consec_write_checks: How many consecutive checks have we had
+    :returns: (int, int) -- num_consec_read_checks, num_consec_write_checks
     """
     if get_global_option('circuit_breaker_url'):
         if circuit_breaker.is_open():
             logger.warning('Circuit breaker is OPEN!')
-            return None
+            return (0, 0)
 
     try:
-        read_update_needed, updated_read_units = __ensure_provisioning_reads(
-            table_name, key_name)
-        write_update_needed, updated_write_units = __ensure_provisioning_writes(
-            table_name, key_name)
+        read_update_needed, updated_read_units, num_consec_read_checks = \
+            __ensure_provisioning_reads(
+                table_name,
+                key_name,
+                num_consec_read_checks)
+        write_update_needed, updated_write_units, num_consec_write_checks = \
+            __ensure_provisioning_writes(
+                table_name,
+                key_name,
+                num_consec_write_checks)
+
+        if read_update_needed:
+            num_consec_read_checks = 0
+
+        if write_update_needed:
+            num_consec_write_checks = 0
 
         # Handle throughput updates
         if read_update_needed or write_update_needed:
@@ -39,9 +59,9 @@ def ensure_provisioning(table_name, key_name):
                     int(updated_write_units)))
             __update_throughput(
                 table_name,
+                key_name,
                 updated_read_units,
-                updated_write_units,
-                key_name)
+                updated_write_units)
         else:
             logger.info('{0} - No need to change provisioning'.format(
                 table_name))
@@ -50,6 +70,8 @@ def ensure_provisioning(table_name, key_name):
     except BotoServerError:
         raise
 
+    return num_consec_read_checks, num_consec_write_checks
+
 
 def __calculate_always_decrease_rw_values(
         table_name, read_units, provisioned_reads,
@@ -94,23 +116,26 @@ def __calculate_always_decrease_rw_values(
     return (read_units, write_units)
 
 
-def __ensure_provisioning_reads(table_name, key_name):
+def __ensure_provisioning_reads(table_name, key_name, num_consec_read_checks):
     """ Ensure that provisioning is correct
 
     :type table_name: str
     :param table_name: Name of the DynamoDB table
     :type key_name: str
     :param key_name: Configuration option key name
-    :returns: (bool, int) -- update_needed, updated_read_units
+    :type num_consec_read_checks: int
+    :param num_consec_read_checks: How many consecutive checks have we had
+    :returns: (bool, int, int)
+        update_needed, updated_read_units, num_consec_read_checks
     """
     if not get_table_option(key_name, 'enable_reads_autoscaling'):
         logger.info(
             '{0} - Autoscaling of reads has been disabled'.format(table_name))
-        return False, dynamodb.get_provisioned_table_read_units(table_name)
+        return False, dynamodb.get_provisioned_table_read_units(table_name), 0
 
     update_needed = False
     try:
-        updated_read_units = dynamodb.get_provisioned_table_read_units(
+        current_read_units = dynamodb.get_provisioned_table_read_units(
             table_name)
         consumed_read_units_percent = \
             table_stats.get_consumed_read_units_percent(table_name)
@@ -132,15 +157,36 @@ def __ensure_provisioning_reads(table_name, key_name):
             get_table_option(key_name, 'decrease_reads_unit')
         max_provisioned_reads = \
             get_table_option(key_name, 'max_provisioned_reads')
+        num_read_checks_before_scale_down = \
+            get_table_option(key_name, 'num_read_checks_before_scale_down')
+        num_read_checks_reset_percent = \
+            get_table_option(key_name, 'num_read_checks_reset_percent')
     except JSONResponseError:
         raise
     except BotoServerError:
         raise
 
+    # Set the updated units to the current read unit value
+    updated_read_units = current_read_units
+
+    # Reset consecutive reads if num_read_checks_reset_percent is reached
+    if num_read_checks_reset_percent:
+
+        if consumed_read_units_percent >= num_read_checks_reset_percent:
+
+            logger.info(
+                '{0} - Resetting the number of consecutive '
+                'read checks. Reason: Consumed percent {1} is '
+                'greater than reset percent: {2}'.format(
+                    table_name,
+                    consumed_read_units_percent,
+                    num_read_checks_reset_percent))
+
+            num_consec_read_checks = 0
+
     if (consumed_read_units_percent == 0 and not
             get_table_option(
                 key_name, 'allow_scaling_down_reads_on_0_percent')):
-        print('1')
         logger.info(
             '{0} - Scaling down reads is not done when usage is at 0%'.format(
                 table_name))
@@ -148,89 +194,111 @@ def __ensure_provisioning_reads(table_name, key_name):
     elif consumed_read_units_percent >= reads_upper_threshold:
 
         if increase_reads_unit == 'percent':
-            updated_provisioning = calculators.increase_reads_in_percent(
-                updated_read_units,
+            calculated_provisioning = calculators.increase_reads_in_percent(
+                current_read_units,
                 increase_reads_with,
                 get_table_option(key_name, 'max_provisioned_reads'),
                 table_name)
         else:
-            updated_provisioning = calculators.increase_reads_in_units(
-                updated_read_units,
+            calculated_provisioning = calculators.increase_reads_in_units(
+                current_read_units,
                 increase_reads_with,
                 get_table_option(key_name, 'max_provisioned_reads'),
                 table_name)
 
-        if updated_read_units != updated_provisioning:
+        if current_read_units != calculated_provisioning:
+            logger.info(
+                '{0} - Resetting the number of consecutive '
+                'read checks. Reason: scale up event detected'.format(
+                    table_name))
+            num_consec_read_checks = 0
             update_needed = True
-            updated_read_units = updated_provisioning
+            updated_read_units = calculated_provisioning
 
     elif throttled_read_count > throttled_reads_upper_threshold:
 
         if throttled_reads_upper_threshold > 0:
             if increase_reads_unit == 'percent':
-                updated_provisioning = calculators.increase_reads_in_percent(
+                calculated_provisioning = calculators.increase_reads_in_percent(
                     updated_read_units,
                     increase_reads_with,
                     get_table_option(key_name, 'max_provisioned_reads'),
                     table_name)
             else:
-                updated_provisioning = calculators.increase_reads_in_units(
+                calculated_provisioning = calculators.increase_reads_in_units(
                     updated_read_units,
                     increase_reads_with,
                     get_table_option(key_name, 'max_provisioned_reads'),
                     table_name)
 
-            if updated_read_units != updated_provisioning:
+            if current_read_units != calculated_provisioning:
+                logger.info(
+                    '{0} - Resetting the number of consecutive '
+                    'read checks. Reason: scale up event detected'.format(
+                        table_name))
+                num_consec_read_checks = 0
                 update_needed = True
-                updated_read_units = updated_provisioning
+                updated_read_units = calculated_provisioning
 
     elif consumed_read_units_percent <= reads_lower_threshold:
 
         if decrease_reads_unit == 'percent':
-            updated_provisioning = calculators.decrease_reads_in_percent(
+            calculated_provisioning = calculators.decrease_reads_in_percent(
                 updated_read_units,
                 decrease_reads_with,
                 get_table_option(key_name, 'min_provisioned_reads'),
                 table_name)
         else:
-            updated_provisioning = calculators.decrease_reads_in_units(
+            calculated_provisioning = calculators.decrease_reads_in_units(
                 updated_read_units,
                 decrease_reads_with,
                 get_table_option(key_name, 'min_provisioned_reads'),
                 table_name)
 
-        if updated_read_units != updated_provisioning:
-            update_needed = True
-            updated_read_units = updated_provisioning
+        if current_read_units != calculated_provisioning:
+            num_consec_read_checks = num_consec_read_checks + 1
+
+            if num_consec_read_checks >= num_read_checks_before_scale_down:
+                update_needed = True
+                updated_read_units = calculated_provisioning
 
     if max_provisioned_reads:
-        if (int(updated_read_units) > int(max_provisioned_reads)):
+        if int(updated_read_units) > int(max_provisioned_reads):
             update_needed = True
             updated_read_units = int(max_provisioned_reads)
             logger.info(
                 'Will not increase writes over max-provisioned-reads '
                 'limit ({0} writes)'.format(updated_read_units))
 
-    return update_needed, int(updated_read_units)
+    logger.info('{0} - Consecutive read checks {1}/{2}'.format(
+        table_name,
+        num_consec_read_checks,
+        num_read_checks_before_scale_down))
+
+    return update_needed, updated_read_units, num_consec_read_checks
 
 
-def __ensure_provisioning_writes(table_name, key_name):
+def __ensure_provisioning_writes(
+        table_name, key_name, num_consec_write_checks):
     """ Ensure that provisioning of writes is correct
 
     :type table_name: str
     :param table_name: Name of the DynamoDB table
     :type key_name: str
     :param key_name: Configuration option key name
-    :returns: (bool, int) -- update_needed, updated_write_units
+    :type num_consec_write_checks: int
+    :param num_consec_write_checks: How many consecutive checks have we had
+    :returns: (bool, int, int)
+        update_needed, updated_write_units, num_consec_write_checks
     """
     if not get_table_option(key_name, 'enable_writes_autoscaling'):
         logger.info(
             '{0} - Autoscaling of reads has been disabled'.format(table_name))
-        return False, dynamodb.get_provisioned_table_write_units(table_name)
+        return False, dynamodb.get_provisioned_table_write_units(table_name), 0
 
     update_needed = False
     try:
-        updated_write_units = dynamodb.get_provisioned_table_write_units(
+        current_write_units = dynamodb.get_provisioned_table_write_units(
             table_name)
         consumed_write_units_percent = \
             table_stats.get_consumed_write_units_percent(table_name)
@@ -252,11 +320,33 @@ def __ensure_provisioning_writes(table_name, key_name):
             get_table_option(key_name, 'decrease_writes_with')
         max_provisioned_writes = \
             get_table_option(key_name, 'max_provisioned_writes')
+        num_write_checks_before_scale_down = \
+            get_table_option(key_name, 'num_write_checks_before_scale_down')
+        num_write_checks_reset_percent = \
+            get_table_option(key_name, 'num_write_checks_reset_percent')
     except JSONResponseError:
         raise
     except BotoServerError:
         raise
 
+    # Set the updated units to the current read unit value
+    updated_write_units = current_write_units
+
+    # Reset consecutive write count num_write_checks_reset_percent is reached
+    if num_write_checks_reset_percent:
+
+        if consumed_write_units_percent >= num_write_checks_reset_percent:
+
+            logger.info(
+                '{0} - Resetting the number of consecutive '
+                'write checks. Reason: Consumed percent {1} is '
+                'greater than reset percent: {2}'.format(
+                    table_name,
+                    consumed_write_units_percent,
+                    num_write_checks_reset_percent))
+
+            num_consec_write_checks = 0
+
     # Check if we should update write provisioning
     if (consumed_write_units_percent == 0 and not
             get_table_option(
@@ -269,60 +359,74 @@ def __ensure_provisioning_writes(table_name, key_name):
     elif consumed_write_units_percent >= writes_upper_threshold:
 
         if increase_writes_unit == 'percent':
-            updated_provisioning = calculators.increase_writes_in_percent(
-                updated_write_units,
+            calculated_provisioning = calculators.increase_writes_in_percent(
+                current_write_units,
                 increase_writes_with,
                 get_table_option(key_name, 'max_provisioned_writes'),
                 table_name)
         else:
-            updated_provisioning = calculators.increase_writes_in_units(
-                updated_write_units,
+            calculated_provisioning = calculators.increase_writes_in_units(
+                current_write_units,
                 increase_writes_with,
                 get_table_option(key_name, 'max_provisioned_writes'),
                 table_name)
 
-        if updated_write_units != updated_provisioning:
+        if current_write_units != calculated_provisioning:
+            logger.info(
+                '{0} - Resetting the number of consecutive '
+                'write checks. Reason: scale up event detected'.format(
+                    table_name))
+            num_consec_write_checks = 0
             update_needed = True
-            updated_write_units = updated_provisioning
+            updated_write_units = calculated_provisioning
 
     elif throttled_write_count > throttled_writes_upper_threshold:
 
         if throttled_writes_upper_threshold > 0:
             if increase_writes_unit == 'percent':
-                updated_provisioning = calculators.increase_writes_in_percent(
-                    updated_write_units,
-                    increase_writes_with,
-                    get_table_option(key_name, 'max_provisioned_writes'),
-                    table_name)
+                calculated_provisioning = \
+                    calculators.increase_writes_in_percent(
+                        current_write_units,
+                        increase_writes_with,
+                        get_table_option(key_name, 'max_provisioned_writes'),
+                        table_name)
             else:
-                updated_provisioning = calculators.increase_writes_in_units(
-                    updated_write_units,
+                calculated_provisioning = calculators.increase_writes_in_units(
+                    current_write_units,
                     increase_writes_with,
                     get_table_option(key_name, 'max_provisioned_writes'),
                     table_name)
 
-            if updated_write_units != updated_provisioning:
+            if current_write_units != calculated_provisioning:
+                logger.info(
+                    '{0} - Resetting the number of consecutive '
+                    'write checks. Reason: scale up event detected'.format(
+                        table_name))
+                num_consec_write_checks = 0
                 update_needed = True
-                updated_write_units = updated_provisioning
+                updated_write_units = calculated_provisioning
 
     elif consumed_write_units_percent <= writes_lower_threshold:
 
         if decrease_writes_unit == 'percent':
-            updated_provisioning = calculators.decrease_writes_in_percent(
-                updated_write_units,
+            calculated_provisioning = calculators.decrease_writes_in_percent(
+                current_write_units,
                 decrease_writes_with,
                 get_table_option(key_name, 'min_provisioned_writes'),
                 table_name)
         else:
-            updated_provisioning = calculators.decrease_writes_in_units(
-                updated_write_units,
+            calculated_provisioning = calculators.decrease_writes_in_units(
+                current_write_units,
                 decrease_writes_with,
                 get_table_option(key_name, 'min_provisioned_writes'),
                 table_name)
 
-        if updated_write_units != updated_provisioning:
-            update_needed = True
-            updated_write_units = updated_provisioning
+        if current_write_units != calculated_provisioning:
+            num_consec_write_checks = num_consec_write_checks + 1
+
+            if num_consec_write_checks >= num_write_checks_before_scale_down:
+                update_needed = True
+                updated_write_units = calculated_provisioning
 
     if max_provisioned_writes:
         if int(updated_write_units) > int(max_provisioned_writes):
@@ -332,20 +436,25 @@ def __ensure_provisioning_writes(table_name, key_name):
                 'Will not increase writes over max-provisioned-writes '
                 'limit ({0} writes)'.format(updated_write_units))
 
-    return update_needed, int(updated_write_units)
+    logger.info('{0} - Consecutive write checks {1}/{2}'.format(
+        table_name,
+        num_consec_write_checks,
+        num_write_checks_before_scale_down))
+
+    return update_needed, updated_write_units, num_consec_write_checks
 
 
-def __update_throughput(table_name, read_units, write_units, key_name):
+def __update_throughput(table_name, key_name, read_units, write_units):
     """ Update throughput on the DynamoDB table
 
     :type table_name: str
     :param table_name: Name of the DynamoDB table
+    :type key_name: str
+    :param key_name: Configuration option key name
     :type read_units: int
     :param read_units: New read unit provisioning
     :type write_units: int
     :param write_units: New write unit provisioning
-    :type key_name: str
-    :param key_name: Configuration option key name
     """
     try:
         current_ru = dynamodb.get_provisioned_table_read_units(table_name)
